﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;
using LoaderLib;
using System.Windows.Forms;
using System.Reflection;

namespace MainClient
{
    class ModuleManager
    {
        static Dictionary<string, ModuleInfo> _modules = new Dictionary<string, ModuleInfo>();

        public static bool LoadModules(string modules_dir, out List<string> modules_name)
        {
            _modules.Clear();
            modules_name = new List<string>();

            if (!Directory.Exists(modules_dir))
            {
                Directory.CreateDirectory(modules_dir);
                return true;
            }

            string load_config = modules_dir + "\\Modules.xml";
            if (!File.Exists(load_config))
                return false;
            try
            {
                XmlDocument xd = new XmlDocument();
                xd.Load(load_config);

                XmlNodeList xnl = xd.GetElementsByTagName("Module");
                foreach (XmlNode xn in xnl)
                {
                    string name = xn.Attributes.GetNamedItem("name").Value;
                    string file = xn.Attributes.GetNamedItem("file").Value;
                    string interface_name = xn.Attributes.GetNamedItem("interface_name").Value;

                    ModuleInfo moduleInfo = new ModuleInfo(name, file, interface_name);
                    if (_modules.ContainsKey(name))
                        continue;

                    _modules.Add(name, moduleInfo);
                    modules_name.Add(name);
                }

                return true;
            }
            catch
            {
                return false;
            }
        }

        public static Form LoadModule(string moduleName)
        {
            if (!_modules.ContainsKey(moduleName))
                return null;

            ModuleInfo mInfo = _modules[moduleName];
            if (mInfo.Frm_Module != null && !mInfo.Frm_Module.IsDisposed)
                return mInfo.Frm_Module;

            byte[] bFile = File.ReadAllBytes(mInfo.File);
            Assembly amy = Assembly.Load(bFile);
            ILoader loader = (ILoader)amy.CreateInstance(mInfo.Interface_Name);
            Form frm = loader.LoadModule(mInfo.File);
            mInfo.Frm_Module = frm;
            return frm;
        }
    }

    class ModuleInfo
    {
        string name;
        /// <summary>
        /// 模块唯一标识，也作为应用程序域的名称
        /// </summary>
        public string Name
        {
            get { return name; }
        }

        string file;
        /// <summary>
        /// 模块对应的文件名
        /// </summary>
        public string File
        {
            get { return file; }
            set { file = value; }
        }

        string interface_name;
        /// <summary>
        /// 用于程序框架加载模块的入口
        /// </summary>
        public string Interface_Name
        {
            get { return interface_name; }
            set { interface_name = value; }
        }

        /// <summary>
        /// 功能模块窗体
        /// </summary>
        public Form Frm_Module;

        public ModuleInfo(string name, string file, string interface_name)
        {
            this.name = name;
            this.file = file;
            this.interface_name = interface_name;
        }
    }
}
